﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Classification.Views {
  public partial class InteractiveSymbolicDiscriminantFunctionClassificationSolutionSimplifierView : InteractiveSymbolicClassificationSolutionSimplifierViewBase {
    private readonly SymbolicDiscriminantFunctionClassificationSolutionImpactValuesCalculator calculator;

    public new SymbolicDiscriminantFunctionClassificationSolution Content {
      get { return (SymbolicDiscriminantFunctionClassificationSolution)base.Content; }
      set { base.Content = value; }
    }

    public InteractiveSymbolicDiscriminantFunctionClassificationSolutionSimplifierView()
      : base() {
      InitializeComponent();
      this.Caption = "Interactive Classification Solution Simplifier";

      calculator = new SymbolicDiscriminantFunctionClassificationSolutionImpactValuesCalculator();
    }

    protected override void UpdateModel(ISymbolicExpressionTree tree) {
      var model = CreateModel(tree);
      model.RecalculateModelParameters(Content.ProblemData, Content.ProblemData.TrainingIndices);
      Content.Model = (ISymbolicDiscriminantFunctionClassificationModel)model;
    }

    protected override Dictionary<ISymbolicExpressionTreeNode, double> CalculateReplacementValues(ISymbolicExpressionTree tree) {
      return calculator.CalculateReplacementValues(tree, Content.Model.Interpreter, Content.ProblemData).ToDictionary(x => x.Item1, x => x.Item2);
    }

    protected override Dictionary<ISymbolicExpressionTreeNode, double> CalculateImpactValues(ISymbolicExpressionTree tree) {
      return calculator.CalculateImpactValues(tree, Content.Model.Interpreter, Content.ProblemData,
                                              Content.Model.LowerEstimationLimit, Content.Model.UpperEstimationLimit).ToDictionary(x => x.Item1, x => x.Item2); ;
    }

    protected override void btnOptimizeConstants_Click(object sender, EventArgs e) {

    }
  }
}
